﻿using System;
using System.Diagnostics;
using System.Threading;
using System.Web;
using System.Web.Configuration;
using System.Web.UI;

namespace REFOR.BL
{

    /// <summary>
    /// Funciones genericas
    /// </summary>
    public static class Genericas
    {
        #region Funciones para control Refresh

        // Constantes
        private const string KEY_REF_DEC_HID = "RefDecHid";
        private const string KEY_REF_DEC_STAMP = "RefDecStamp";

        /// <summary>
        /// Determines whether [is page refresh].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is page refresh]; otherwise, <c>false</c>.
        /// </returns>
        public static bool IsPageRefresh()
        {
            Page page = HttpContext.Current.CurrentHandler as Page;

            string val = HttpContext.Current.Request[KEY_REF_DEC_HID];

            if ((val == null) || (val.Length == 0)) return false;

            string stamp = HttpContext.Current.Session[KEY_REF_DEC_STAMP] as string;
            if (stamp == null)
            {
                HttpContext.Current.Response.Clear();
                HttpContext.Current.Response.Write("Session Expired.");
                HttpContext.Current.Response.End();
            }
            if (stamp != val)
                return true;
            else
                return false;
        }
        /// <summary>
        /// Stamps the refresh detection.
        /// </summary>
        /// You need to call this method in Page_PreRender to make the
        /// refresh detection working
        public static void StampRefreshDetection()
        {
            Page page = HttpContext.Current.CurrentHandler as Page;

            string stamp = HttpContext.Current.Server.UrlEncode(DateTime.Now.ToString());

            HttpContext.Current.Session[KEY_REF_DEC_STAMP] = stamp;

            page.ClientScript.RegisterHiddenField(KEY_REF_DEC_HID, stamp);
        }

        #endregion Refresh

        /// <summary>
        /// Este método permite registrar el error que se produce en la Aplicación en el registro de Eventos del Servidor
        /// En caso de existir algún problema para salvar el error en la tabla GR_ERROR, se registrará en el Registro de Windows del Servidor.
        /// </summary>
        /// <param name="eventDescription">Información de la Excepción capturada. Describe la operación que se ha intentado realizar y que ha provocado el error.</param>
        /// <param name="eventID">ID asignado al evento para ser guardado en el Registro de Windows. Permite filtar por tipo de Evento.</param>
        public static void SetEvent(string eventDescription, int eventID)
        {
            // Instancia del EventLog para tener accesos al registro de Windows.
            EventLog eventLog = new EventLog();

            string applicationName = WebConfigurationManager.AppSettings["ApplicationName"].ToString();
            if (applicationName == null)
                applicationName = WebConfigurationManager.AppSettings["NombreAplicacion"].ToString();
            string windowsEventTag = WebConfigurationManager.AppSettings["LogEvent"].ToString();
            string userID = Thread.CurrentPrincipal.Identity.Name;

            // LogEvent parameters
            eventLog.Source = applicationName;
            eventLog.Log = windowsEventTag; // Usually 'Application'

            try
            {
                eventLog.WriteEntry(QuitarHtml(eventDescription), System.Diagnostics.EventLogEntryType.Error, eventID);
            }
            catch
            {
                return;   // anulamos los posibles errores para que no entre en un bucle
            }

        }

        /// <summary>
        /// Manejo de un error inesperado.
        /// </summary>
        /// <param name="?">La descripcion del evento.</param>
        /// <param name="err">La excepción.</param>
        /// <param name="pagina">La página que produjo el error.</param>
        public static void ErrorInesperado(Exception err, Page pagina)
        {
            // Creamos un objeto Server para poder tomar los datos del error
            HttpServerUtility Server = HttpContext.Current.Server;


            string eventDescription = "Error inesperado en la página '" + pagina.Title.Trim() + "'.";
            string applicationName = WebConfigurationManager.AppSettings["ApplicationName"].ToString();
            if (applicationName == null)
                applicationName = WebConfigurationManager.AppSettings["NombreAplicacion"].ToString();
            string windowsEventTag = WebConfigurationManager.AppSettings["LogEvent"].ToString();


            //Guardamos el error en el registro de Eventos de Windows
            SetEvent(eventDescription + err.Message, 90);

        }

        /// <summary>
        /// Se ha producido un error inesperado.
        /// </summary>
        public static void ErrorInesperado()
        {
            // Creamos un objeto Server para poder tomar los datos del error
            HttpServerUtility Server = HttpContext.Current.Server;

            Exception err = Server.GetLastError().GetBaseException();
            // y creamos un objeto Page para poder mostrar los datos del entorno
            Page pagina = HttpContext.Current.Handler as Page;

            string eventDescription;

            if (pagina != null)
                eventDescription = "Error inesperado en la página '" + pagina.Title.Trim() + "'.\r\n";
            else
                if (Server.GetLastError().GetType() == typeof(HttpParseException) && ((System.Web.HttpParseException)Server.GetLastError()).FileName != null)
                    eventDescription = "Error inesperado en la página '" + ((System.Web.HttpParseException)Server.GetLastError()).FileName.Trim() + "'.\r\n";
                else
                    eventDescription = "Error inesperado en página desconocida.\r\n";

            // tomamos el nombre de la aplicacion del web.config ya que no ha sido posible sacarla de ningun objeto
            string applicationName = WebConfigurationManager.AppSettings["ApplicationName"].ToString();
            if (applicationName == null)
                applicationName = WebConfigurationManager.AppSettings["NombreAplicacion"].ToString();
            string windowsEventTag = WebConfigurationManager.AppSettings["LogEvent"].ToString();

            //Guardamos el error en el registro de Eventos de Windows
            SetEvent(eventDescription + err.Message, 90);

            // y en cualquier caso, nos vamos a la pagina de error.
            Server.Transfer(@"~\ErrorPages\Oops.aspx");

        }

        /// <summary>
        /// Manejo de un error inesperado.
        /// </summary>
        /// <param name="entorno">The entorno.</param>
        /// <param name="eventDescription">The event description.</param>
        /// <param name="err">La excepcion.</param>
        public static void ErrorInesperado(string eventDescription, Exception err, string tituloPagina)
        {
            string applicationName = WebConfigurationManager.AppSettings["ApplicationName"].ToString();
            if (applicationName == null)
                applicationName = WebConfigurationManager.AppSettings["NombreAplicacion"].ToString();
            string windowsEventTag = WebConfigurationManager.AppSettings["LogEvent"].ToString();
            string userID = Thread.CurrentPrincipal.Identity.Name;


            //Guardamos el error en el registro de Eventos de Windows
            SetEvent(eventDescription + err.Message, 90);

        }


        /// <summary>
        /// Quitar los codigos HTML.
        /// </summary>
        /// <param name="cadenaParaLimpiar">La cadena para limpiar.</param>
        /// <returns>Una cadena sin codigos extraños</returns>
        private static string QuitarHtml(string cadenaParaLimpiar)
        {

            cadenaParaLimpiar = cadenaParaLimpiar.Replace("<br>", "\r\n");
            cadenaParaLimpiar = cadenaParaLimpiar.Replace("</br>", "\r\n");
            cadenaParaLimpiar = cadenaParaLimpiar.Replace("<br/>", "\r\n");
            cadenaParaLimpiar = cadenaParaLimpiar.Replace("<br />", "\r\n");
            cadenaParaLimpiar = cadenaParaLimpiar.Replace("<FONT COLOR='Red'>", "");
            cadenaParaLimpiar = cadenaParaLimpiar.Replace("<FONT COLOR='DarkBlue'>", "");
            cadenaParaLimpiar = cadenaParaLimpiar.Replace("</FONT>", "");

            return cadenaParaLimpiar;
        }

    }
}
