﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class DatoPropioRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="DatoPropioID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public DatoPropio ObtenerUnico()
        {
            List<DatoPropio> lista;
            DbSet<DatoPropio> query = context.Set<DatoPropio>();
            lista = query.Include("Municipio").Include("Municipio.Provincia").ToList();

            return lista.First();
        }


        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<VistaDatosPropios> GetDatosPropiosParaReporte()
        {
            try
            {
                return context.VistaDatosPropios.ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="DatoPropioID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(string NIF, string Responsable, string Denominacion, string Domicilio, string Codigo_INE, string Codigo_postal,
            string AliasTelefonoUno, string TelefonoUno, string AliasTelefonoDos, string TelefonoDos, string AliasTelefonoTres, string TelefonoTres,
            string CorreoElectronico, string Web, string DomicilioEntidadBancaria, string IBAN, int MesesCambioContraseña, int IVA, int ID)
        {
            DatoPropio datoPropio = ObtenerUnico();
            if (datoPropio == null)
                return 0;

            datoPropio.NIF = NIF.ToUpper();
            datoPropio.Responsable = Responsable;
            datoPropio.Denominacion = Denominacion;
            datoPropio.Domicilio = Domicilio;
            datoPropio.Codigo_INE = Codigo_INE;
            datoPropio.Codigo_postal = Codigo_postal;
            datoPropio.AliasTelefonoUno = AliasTelefonoUno;
            datoPropio.TelefonoUno = TelefonoUno;
            datoPropio.AliasTelefonoDos = AliasTelefonoDos;
            datoPropio.TelefonoDos = TelefonoDos;
            datoPropio.AliasTelefonoTres = AliasTelefonoTres;
            datoPropio.TelefonoTres = TelefonoTres;
            datoPropio.CorreoElectronico = CorreoElectronico;
            datoPropio.Web = Web;
            datoPropio.DomicilioEntidadBancaria = DomicilioEntidadBancaria;
            datoPropio.IBAN = (IBAN == null) ? IBAN : IBAN.ToUpper();
            datoPropio.MesesCambioContraseña = MesesCambioContraseña;
            datoPropio.IVA = IVA;
            
            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
