﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class FacturaRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Factura> GetFacturas(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                    DbSet<Factura> query = context.Set<Factura>();
                    if (OrdenarPor == "facturaID")
                        return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FacturaID).ToList();
                    else if (OrdenarPor == "cliente")
                        return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Cliente.Denominacion).ToList();
                    else if (OrdenarPor == "fechaEmision")
                        return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FechaEmision).ToList();
                    else if (OrdenarPor == "emitida")
                        return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Emitida).ToList();
                    else
                        return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();

                //return query.ToList();
                    //return context.Facturas.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Factura> GetFacturasEmitidasPendientesDePago(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                DbSet<Factura> query = context.Set<Factura>();
                if (OrdenarPor == "facturaID")
                    return query.Include("Cliente").Where(r => r.Emitida && r.ImporteTotal != r.ImportePagado).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FacturaID).ToList();
                else if (OrdenarPor == "cliente")
                    return query.Include("Cliente").Where(r => r.Emitida && r.ImporteTotal != r.ImportePagado).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Cliente.Denominacion).ToList();
                else if (OrdenarPor == "fechaEmision")
                    return query.Include("Cliente").Where(r => r.Emitida && r.ImporteTotal != r.ImportePagado).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FechaEmision).ToList();
                else if (OrdenarPor == "emitida")
                    return query.Include("Cliente").Where(r => r.Emitida && r.ImporteTotal != r.ImportePagado).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Emitida).ToList();
                else
                    return query.Include("Cliente").Where(r => r.Emitida && r.ImporteTotal != r.ImportePagado).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();

                //return query.ToList();
                //return context.Facturas.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Factura> GetFacturasSinEmitirConLineas(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                DbSet<Factura> query = context.Set<Factura>();
                if (OrdenarPor == "facturaID")
                    return query.Include("Cliente").Include("LineasFactura").Where(r => !r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FacturaID).ToList();
                else if (OrdenarPor == "cliente")
                    return query.Include("Cliente").Include("LineasFactura").Where(r => !r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Cliente.Denominacion).ToList();
                else if (OrdenarPor == "fechaEmision")
                    return query.Include("Cliente").Include("LineasFactura").Where(r => !r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FechaEmision).ToList();
                else if (OrdenarPor == "emitida")
                    return query.Include("Cliente").Include("LineasFactura").Where(r => !r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Emitida).ToList();
                else
                    return query.Include("Cliente").Include("LineasFactura").Where(r => !r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();
                //return query.ToList();
                //return context.Facturas.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Factura> GetFacturasEmitidasConLineas(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                DbSet<Factura> query = context.Set<Factura>();
                if (OrdenarPor == "facturaID")
                    return query.Include("Cliente").Include("LineasFactura").Where(r => r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FacturaID).ToList();
                else if (OrdenarPor == "cliente")
                    return query.Include("Cliente").Include("LineasFactura").Where(r => r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Cliente.Denominacion).ToList();
                else if (OrdenarPor == "fechaEmision")
                    return query.Include("Cliente").Include("LineasFactura").Where(r => r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FechaEmision).ToList();
                else if (OrdenarPor == "emitida")
                    return query.Include("Cliente").Include("LineasFactura").Where(r => r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Emitida).ToList();
                else
                    return query.Include("Cliente").Include("LineasFactura").Where(r => r.Emitida).Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();
                //return query.ToList();
                //return context.Facturas.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Factura> GetFacturas()
        {
            try
            {
                return context.Facturas.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Factura> GetFacturaConLineas(int FacturaID)
        {
            try
            {
                DbSet<Factura> query = context.Set<Factura>();
                return query.Include("Cliente").Include("LineasFactura").Where(r => r.FacturaID == FacturaID).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<VistaFacturas> GetFacturasParaReporte()
        {
            try
            {
                return context.VistaFacturas.ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<VistaFacturas> GetUnaFacturaParaImprimir(int FacturaID)
        {
            try
            {
                return context.VistaFacturas.Where(r => r.FacturaID == FacturaID).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(int ClienteID, DateTime FechaEmision, int AñoFiscal, int Numero, int PresupuestoID, string Descripcion, decimal ImporteTotal,
            decimal ImportePagado, string ObservacionesCliente, string ObservacionesInternas)
        {
            
            Factura factura = new Factura();
            factura.FacturaID = 0;
            factura.ClienteID = ClienteID;
            factura.FechaEmision = FechaEmision;
            factura.AñoFiscal = AñoFiscal;
            factura.Numero = Numero;
            factura.PresupuestoID = PresupuestoID;
            factura.Descripcion = Descripcion.Trim();
            factura.ImporteTotal = ImporteTotal;
            factura.ImportePagado = ImportePagado;
            factura.ObservacionesCliente = ObservacionesCliente;
            factura.ObservacionesInternas = ObservacionesInternas;

            context.Facturas.Add(factura);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="FacturaID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Factura ObtenerPorID(int FacturaID)
        {
            return context.Facturas.Find(FacturaID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="FacturaID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int FacturaID)
        {
            Factura factura = ObtenerPorID(FacturaID);
            if (factura == null)
                return 0;

            context.Facturas.Remove(factura);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="FacturaID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int FacturaID, int ClienteID, DateTime FechaEmision, string Descripcion, string ObservacionesCliente, string ObservacionesInternas)
        {
           
            Factura factura = ObtenerPorID(FacturaID);
            if (factura == null)
                return 0;

            factura.ClienteID = ClienteID;
            factura.FechaEmision = FechaEmision;
            factura.Descripcion = Descripcion.Trim();
            factura.ObservacionesCliente = ObservacionesCliente;
            factura.ObservacionesInternas = ObservacionesInternas;

            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="FacturaID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar_pago(int FacturaID, decimal ImportePagado)
        {

            Factura factura = ObtenerPorID(FacturaID);
            if (factura == null)
                return 0;

            factura.ImportePagado = ImportePagado;

            return SaveChanges();
        }

        /// <summary>
        /// Generar una factura con los datos del presupuesto aceptado.
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        public void GenerarFacturaDesdePresupuesto(int PresupuestoID)
        {
            Presupuesto presupuesto = (new PresupuestoRepository()).ObtenerPorID(PresupuestoID);
            if (presupuesto == null)
                throw new Exception("Presupuesto no encontrado");

            DatoPropio datoPropio = (new DatoPropioRepository()).ObtenerUnico();

            Factura factura = new Factura();
            factura.FacturaID = 0;
            factura.ClienteID = presupuesto.ClienteID;
            factura.FechaEmision = DateTime.Today;
            factura.AñoFiscal = 0;
            factura.Numero = 0;
            factura.PresupuestoID = PresupuestoID;
            factura.Descripcion = "Factura correspondiente al presupuesto "+PresupuestoID.ToString();
            factura.Importe = 0;
            factura.TipoIVA = datoPropio.IVA;
            factura.ImporteIva = 0;
            factura.ImporteTotal = 0;
            factura.ImportePagado = 0;
            factura.Emitida = false;

            context.Facturas.Add(factura);
            if(SaveChanges() != 1)
                throw new Exception("No se ha podido generar la factura");

            (new LineaFacturaRepository()).Insertar(factura.FacturaID, factura.Descripcion, presupuesto.Importe);

            (new PresupuestoRepository()).MarcarComoFacturado(PresupuestoID);

            return;
        }


        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Recalcula el presupuesto solicitado.
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        public void Recalcular(int FacturaID)
        {
            Factura factura = GetFacturaConLineas(FacturaID).First();

            decimal importe = 0;

            foreach (LineaFactura linea in factura.LineasFactura)
            {
                importe += linea.Importe;
            }

            factura.Importe = Math.Round(importe, 2);
            factura.ImporteIva = Math.Round((factura.Importe * factura.TipoIVA / 100), 2);
            factura.ImporteTotal = factura.Importe + factura.ImporteIva;

            SaveChanges();
        }

        public void Emitir(int FacturaID)
        {
            Factura factura = ObtenerPorID(FacturaID);
            if (factura == null)
                throw new Exception("No se ha encontrado la factura");

            // averiguamos el ultimo numero de factura utilizado en este año
            int numero = Buscar_ultima_factura(DateTime.Today.Year);

            factura.FechaEmision = DateTime.Today;
            factura.Emitida = true;
            factura.AñoFiscal = DateTime.Today.Year;
            factura.Numero = (numero + 1);
            SaveChanges();

        }

        private int Buscar_ultima_factura(int AñoFiscal)
        {
            try
            {
                Factura ultimaFactura = context.Facturas.Where(r => r.AñoFiscal==AñoFiscal).OrderBy(r => r.Numero).ToList().Last();
                return ultimaFactura.Numero;
            }
            catch
            {
                return 0;
            }
        }

        /// <summary>
        /// Recalcula la factura solicitada.
        /// </summary>
        /// <param name="PresupuestoID">The factura identifier.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        public void Reordenar(int FacturaID)
        {
            Factura factura = GetUnaFacturaSinEmitirConLineas(FacturaID).First();

            // lo realizamos en dos pasasadas
            // en la primera le quitamos 1.000.000 al numero para evitar que coincida con ninguno actual
            foreach (LineaFactura linea in factura.LineasFactura.OrderBy(r => r.Orden))
            {
                linea.Orden = linea.Orden - 1000000;
            }
            SaveChanges();

            // y ahora empezamos desde 10
            int orden = 10;

            foreach (LineaFactura linea in factura.LineasFactura.OrderBy(r => r.Orden))
            {
                linea.Orden = orden;
                orden += 10;
            }
            SaveChanges();
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Factura> GetUnaFacturaSinEmitirConLineas(int FacturaID)
        {
            try
            {
                DbSet<Factura> query = context.Set<Factura>();
                return query.Include("Cliente").Include("LineasFactura").Where(r => r.FacturaID == FacturaID).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }


    }
}
