﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class GastoRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Gasto> GetGastos(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                    DbSet<Gasto> query = context.Set<Gasto>();
                    if (OrdenarPor == "gastoID")
                        return query.Include("Proveedor").Include("Rubrica").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.GastoID).ToList();
                    else if (OrdenarPor == "fechaRecepcion")
                        return query.Include("Proveedor").Include("Rubrica").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FechaRecepcion).ToList();
                    else if (OrdenarPor == "proveedor")
                        return query.Include("Proveedor").Include("Rubrica").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Proveedor.Denominacion).ToList();
                    else if (OrdenarPor == "rubrica")
                        return query.Include("Proveedor").Include("Rubrica").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Rubrica.Descripcion).ToList();
                    else
                        return query.Include("Proveedor").Include("Rubrica").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();
                //return query.ToList();
                    //return context.Gastos.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Gasto> GetGastos()
        {
            try
            {
                return context.Gastos.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<VistaGastos> GetGastosParaReporte()
        {
            try
            {
                return context.VistaGastos.ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(DateTime FechaRecepcion, DateTime? FechaEmision, int? ProveedorID, int? AñoFiscal, int? Numero, string Descripcion, 
            decimal Importe, int RubricaGastoID, string Observaciones)
        {
            
            Gasto gasto = new Gasto();
            gasto.GastoID = 0;
            gasto.FechaRecepcion = FechaRecepcion;
            gasto.FechaEmision = FechaEmision;
            gasto.ProveedorID = ProveedorID;
            gasto.AñoFiscal = AñoFiscal;
            gasto.Numero = Numero;
            gasto.Descripcion = Descripcion.Trim();
            gasto.Importe = Importe;
            gasto.RubricaGastoID = RubricaGastoID;
            gasto.Observaciones = Observaciones;

            context.Gastos.Add(gasto);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="GastoID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Gasto ObtenerPorID(int GastoID)
        {
            return context.Gastos.Find(GastoID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="GastoID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int GastoID)
        {
            Gasto gasto = ObtenerPorID(GastoID);
            if (gasto == null)
                return 0;

            context.Gastos.Remove(gasto);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="GastoID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int GastoID, DateTime FechaRecepcion, DateTime? FechaEmision, int? ProveedorID, int? AñoFiscal, int? Numero, string Descripcion,
            decimal Importe, int RubricaGastoID, string Observaciones)
        {
            Gasto gasto = ObtenerPorID(GastoID);
            if (gasto == null)
                return 0;

            gasto.FechaRecepcion = FechaRecepcion;
            gasto.FechaEmision = FechaEmision;
            gasto.ProveedorID = ProveedorID; ;
            gasto.AñoFiscal = AñoFiscal;
            gasto.Numero = Numero;
            gasto.Descripcion = Descripcion.Trim();
            gasto.Importe =  Importe;
            gasto.RubricaGastoID = RubricaGastoID;
            gasto.Observaciones = Observaciones;

            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
