﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class LineaFacturaRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Texto">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<LineaFactura> GetLineasFactura(int FacturaID)
        {
            try
            {
                return context.LineasFactura.Where(r => r.FacturaID == FacturaID).OrderBy(r => r.Orden).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Texto">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(int FacturaID, string Texto, decimal Importe)
        {

            // averiaguamos el ultimo numero de orden utilizado para este presupuesto
            int orden = UltimoNumeroOrden(FacturaID);
            
            LineaFactura lineaFactura = new LineaFactura();
            lineaFactura.LineaID = 0;
            lineaFactura.Orden = orden + 10;
            lineaFactura.FacturaID = FacturaID;
            lineaFactura.Texto = Texto.Trim();
            lineaFactura.Importe = Importe;

            context.LineasFactura.Add(lineaFactura);
            int respuesta = SaveChanges();

            (new FacturaRepository()).Recalcular(FacturaID);

            return respuesta;
        }

        /// <summary>
        /// Ultimoes the numero orden.
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <returns>El ultimo numero de orden utilizado</returns>
        /// <exception cref="System.NotImplementedException"></exception>
        private int UltimoNumeroOrden(int FacturaID)
        {
            try
            {
                LineaFactura ultimaLinea = context.LineasFactura.Where(r => r.FacturaID == FacturaID).OrderBy(r => r.Orden).ToList().Last();
                return ultimaLinea.Orden;
            }
            catch
            {
                return 0;
            }
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="LineaID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public LineaFactura ObtenerPorID(int LineaID)
        {
            return context.LineasFactura.Find(LineaID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="LineaID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int LineaID)
        {
            LineaFactura lineaFactura = ObtenerPorID(LineaID);
            if (lineaFactura == null)
                return 0;

            int FacturaID = lineaFactura.FacturaID;

            context.LineasFactura.Remove(lineaFactura);
            int resultado = SaveChanges();

            (new FacturaRepository()).Recalcular(FacturaID);
            (new FacturaRepository()).Reordenar(FacturaID);

            return resultado;

        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="LineaID">La clave a buscar.</param>
        /// <param name="Texto">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int LineaID, int Orden, string Texto, decimal Importe)
        {
            LineaFactura lineaFactura = ObtenerPorID(LineaID);
            if (lineaFactura == null)
                return 0;

            lineaFactura.Orden = Orden;
            lineaFactura.Texto = Texto.Trim();
            lineaFactura.Importe = Importe;

            int resultado = SaveChanges();

            (new FacturaRepository()).Recalcular(lineaFactura.FacturaID);
            (new FacturaRepository()).Reordenar(lineaFactura.FacturaID);

            return resultado;
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
