﻿using System;
using System.Data;
using System.Data.Common;
using System.Data.Entity.Infrastructure;
using System.Data.Odbc;
using System.Data.OleDb;
using System.Data.SqlClient;

namespace REFOR.DL
{
   public static class MensajeError
    {
        /// <summary>
        /// Crear mensaje personalizado a partir de la excepcion sql.
        /// </summary>
        /// <param name="sqlEx">La excepcion sql</param>
        /// <returns>Mensaje personalizado</returns>
        public static string Crear_mensaje(Exception ex)
        {
            string mensaje = string.Empty;

            if (ex is SqlException)
            {
                if (ex.Message.ToUpper().Replace('"', '\'').Contains("PK_"))
                    mensaje = "No puede haber registros con el mismo identificativo";
            }
            else if (ex is DbUpdateException && ex.InnerException.InnerException.Message.ToUpper().Replace('"', '\'').Contains("PRIMARY KEY"))
                mensaje = "No puede haber dos registros con la misma clave. " + ex.InnerException.InnerException.Message;
            else if (ex is DbUpdateException && ex.InnerException.InnerException.Message.ToUpper().Replace('"', '\'').Contains("UNIQUE KEY"))
                mensaje = "No puede haber dos registros con el mismo dato clave. " + ex.InnerException.InnerException.Message;
            else if (ex is DbUpdateException && ex.InnerException.InnerException.Message.ToUpper().Replace('"', '\'').Contains("FOREIGN KEY"))
                mensaje = "No existe vinculación entre este registro y una tabla relacionada. " + ex.InnerException.InnerException.Message;
            else if (ex is DbUpdateException && ex.InnerException.InnerException.Message.ToUpper().Replace('"', '\'').Contains(" REFERENCE "))
                mensaje = "No existe vinculación entre este registro y una tabla relacionada. " + ex.InnerException.InnerException.Message;
            else if (ex is DbUpdateException)
                mensaje = "No se ha podido efectuar la actualización de la tabla. " + ex.Message;
            else if (ex is EntityCommandCompilationException)
                mensaje = "La query no es correcta. " + ex.Message;
            else if (ex is EntitySqlException)
                mensaje = "El comando tiene un error sintactico o semantico. " + ex.Message;
            else if (ex is MappingException)
                mensaje = "Existe un error de mapping. " + ex.Message;
            else if (ex is MetadataException)
                mensaje = "La aplicación no encuentra el fichero edmx. " + ex.Message;
            else if (ex is ProviderIncompatibleException)
                mensaje = "El proveedor de datos no es compatible. " + ex.Message;
            else if (ex is ConstraintException)
                mensaje = "Una o mas condicines no se cumplen. " + ex.Message;
            else if (ex is DuplicateNameException)
                mensaje = "Existen objetos con nombre duplicado. " + ex.Message;
            else if (ex is EvaluateException)
                mensaje = "No se puede evaluar la expresión. " + ex.Message;
            else if (ex is MissingPrimaryKeyException)
                mensaje = "Existe un error con la clave primaria. " + ex.Message;
            else if (ex is NoNullAllowedException)
                mensaje = "No se puede asignar el valor null a la columna. " + ex.Message;
            else if (ex is ObjectNotFoundException)
                mensaje = "No existe el objeto. " + ex.Message;
            else if (ex is OperationAbortedException)
                mensaje = "Se ha abortado la operación. " + ex.Message;
            else if (ex is PropertyConstraintException)
                mensaje = "Se ha intentado forzar una condición. " + ex.Message;
            else if (ex is ReadOnlyException)
                mensaje = "Se intenta escribir un campo de solo lectura. " + ex.Message;
            else if (ex is StrongTypingException)
                mensaje = "No se puede asignar el valor null a la columna. " + ex.Message;
            else if (ex is SyntaxErrorException)
                mensaje = "Error en la consulta generada o el proveedor no la entiende. " + ex.Message;
            else if (ex is UpdateException)
                mensaje = "Error actualizando la base de datos. " + ex.Message;
            else if (ex is DbException)
                mensaje = "Error con la fuente de datos. " + ex.Message;
            else if (ex is OdbcException)
                mensaje = "Error con la fuente de datos ODBC. " + ex.Message;
            else if (ex is OleDbException)
                mensaje = "Error con la fuente de datos OLE-DB. " + ex.Message;
            else if (ex is OptimisticConcurrencyException)
                mensaje = "Se ha producido un error de concurrencia. " + ex.Message;

            if (mensaje == string.Empty)
                mensaje = ex.Message;

            return mensaje;
        }

    }
}
