﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class MunicipioRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Nombre">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Municipio> GetMunicipios(string Nombre, string OrdenarPor)
        {
            Nombre = (Nombre == null || Nombre.Trim() == string.Empty) ? string.Empty : Nombre.Trim();

            try
            {
                    DbSet<Municipio> query = context.Set<Municipio>();
                    if (OrdenarPor == "municipio")
                        return query.Include("Provincia").Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.MunicipioID).ToList();
                    else if (OrdenarPor == "provincia")
                        return query.Include("Provincia").Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.Provincia.Descripcion).ToList();
                    else if (OrdenarPor == "codigo_INE")
                        return query.Include("Provincia").Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.Codigo_INE).ToList();
                    else
                        return query.Include("Provincia").Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.Nombre).ToList();

                //return query.ToList();
                    //return context.Municipios.Where(r => r.Nombre.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Nombre).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Nombre">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Municipio> GetMunicipios()
        {
            try
            {
                return context.Municipios.OrderBy(r => r.Nombre).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(string Codigo_INE, string MunicipioID,string ProvinciaID, string Nombre)
        {
            Municipio municipio = new Municipio();
            municipio.Codigo_INE = Codigo_INE;
            municipio.MunicipioID = MunicipioID;
            municipio.ProvinciaID = ProvinciaID;
            municipio.Nombre = Nombre;

            context.Municipios.Add(municipio);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="MunicipioID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Municipio ObtenerPorID(string Codigo_INE)
        {
            return context.Municipios.Find(Codigo_INE);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="MunicipioID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(string Codigo_INE)
        {
            Municipio municipio = ObtenerPorID(Codigo_INE);
            if (municipio == null)
                return 0;

            context.Municipios.Remove(municipio);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="MunicipioID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(string Codigo_INE, string MunicipioID, string ProvinciaID, string Nombre)
        {
            Municipio municipio = ObtenerPorID(Codigo_INE);
            if (municipio == null)
                return 0;

            municipio.MunicipioID = MunicipioID;
            municipio.ProvinciaID = ProvinciaID;
            municipio.Nombre = Nombre;
            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
