﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class PresupuestoRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Presupuesto> GetPresupuestos(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                DbSet<Presupuesto> query = context.Set<Presupuesto>();
                if (OrdenarPor == "presupuestoID")
                    return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.PresupuestoID).ToList();
                else if (OrdenarPor == "fechaCreacion")
                    return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.FechaCreacion).ToList();
                else if (OrdenarPor == "cliente")
                    return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Cliente.Denominacion).ToList();
                else if (OrdenarPor == "aceptado")
                    return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Aceptado).ToList();
                else
                    return query.Include("Cliente").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();

                //return query.ToList();
                //return context.Presupuestos.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Presupuesto> GetPresupuestosSinAceptarConLineas(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                DbSet<Presupuesto> query = context.Set<Presupuesto>();
                if (OrdenarPor == "presupuestoID")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == false).OrderBy(r => r.PresupuestoID).ToList();
                else if (OrdenarPor == "fechaCreacion")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == false).OrderBy(r => r.FechaCreacion).ToList();
                else if (OrdenarPor == "cliente")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == false).OrderBy(r => r.Cliente.Denominacion).ToList();
                else if (OrdenarPor == "aceptado")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == false).OrderBy(r => r.Aceptado).ToList();
                else
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == false).OrderBy(r => r.Descripcion).ToList();

                //return query.ToList();
                //return context.Presupuestos.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Presupuesto> GetPresupuestosSinFacturarConLineas(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                DbSet<Presupuesto> query = context.Set<Presupuesto>();
                if (OrdenarPor == "presupuestoID")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Facturado == false).OrderBy(r => r.PresupuestoID).ToList();
                else if (OrdenarPor == "fechaCreacion")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Facturado == false).OrderBy(r => r.FechaCreacion).ToList();
                else if (OrdenarPor == "cliente")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Facturado == false).OrderBy(r => r.Cliente.Denominacion).ToList();
                else if (OrdenarPor == "aceptado")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Facturado == false).OrderBy(r => r.Aceptado).ToList();
                else
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Facturado == false).OrderBy(r => r.Descripcion).ToList();

                //return query.ToList();
                //return context.Presupuestos.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Presupuesto> GetPresupuestosAceptadosSinFacturarConLineas(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                DbSet<Presupuesto> query = context.Set<Presupuesto>();
                if (OrdenarPor == "presupuestoID")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == true && r.Facturado == false).OrderBy(r => r.PresupuestoID).ToList();
                else if (OrdenarPor == "fechaCreacion")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == true && r.Facturado == false).OrderBy(r => r.FechaCreacion).ToList();
                else if (OrdenarPor == "cliente")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == true && r.Facturado == false).OrderBy(r => r.Cliente.Denominacion).ToList();
                else if (OrdenarPor == "aceptado")
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == true && r.Facturado == false).OrderBy(r => r.Aceptado).ToList();
                else
                    return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).Where(r => r.Aceptado == true && r.Facturado == false).OrderBy(r => r.Descripcion).ToList();

                //return query.ToList();
                //return context.Presupuestos.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Presupuesto> GetUnPresupuestoSinAceptarConLineas(int PresupuestoID)
        {
            try
            {
                DbSet<Presupuesto> query = context.Set<Presupuesto>();
                return query.Include("Cliente").Include("LineasPresupuestos").Where(r => r.PresupuestoID == PresupuestoID).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<VistaPresupuestos> GetUnPresupuestosParaImprimir(int PresupuestoID)
        {
            try
            {
                return context.VistaPresupuestos.Where(r => r.PresupuestoID == PresupuestoID).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }
        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Presupuesto> GetPresupuestos()
        {
            try
            {
                return context.Presupuestos.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<VistaPresupuestos> GetPresupuestosParaReporte()
        {
            try
            {
                return context.VistaPresupuestos.ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(int ClienteID, DateTime FechaCreacion, DateTime ValidoHasta, string Descripcion,
            string ObservacionesCliente, string ObservacionesInternas, int DescuentoAplicado)
        {

            Presupuesto presupuesto = new Presupuesto();
            presupuesto.PresupuestoID = 0;
            presupuesto.ClienteID = ClienteID;
            presupuesto.FechaCreacion = FechaCreacion;
            presupuesto.ValidoHasta = ValidoHasta;
            presupuesto.Descripcion = Descripcion.Trim();
            presupuesto.ObservacionesCliente = ObservacionesCliente;
            presupuesto.ObservacionesInternas = ObservacionesInternas;
            presupuesto.DescuentoAplicado = DescuentoAplicado;

            context.Presupuestos.Add(presupuesto);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="PresupuestoID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Presupuesto ObtenerPorID(int PresupuestoID)
        {
            return context.Presupuestos.Find(PresupuestoID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="PresupuestoID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int PresupuestoID)
        {
            Presupuesto presupuesto = ObtenerPorID(PresupuestoID);
            if (presupuesto == null)
                return 0;

            context.Presupuestos.Remove(presupuesto);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="PresupuestoID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int PresupuestoID, int ClienteID, DateTime FechaCreacion, DateTime ValidoHasta, string Descripcion,
            string ObservacionesCliente, string ObservacionesInternas, int DescuentoAplicado)
        {
            Presupuesto presupuesto = ObtenerPorID(PresupuestoID);
            if (presupuesto == null)
                return 0;

            presupuesto.ClienteID = ClienteID;
            presupuesto.FechaCreacion = FechaCreacion;
            presupuesto.ValidoHasta = ValidoHasta;
            presupuesto.Descripcion = Descripcion.Trim();
            presupuesto.ObservacionesCliente = ObservacionesCliente;
            presupuesto.ObservacionesInternas = ObservacionesInternas;
            presupuesto.DescuentoAplicado = DescuentoAplicado;


            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Recalcula el presupuesto solicitado.
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        public void Recalcular(int PresupuestoID)
        {
            Presupuesto presupuesto = GetUnPresupuestoSinAceptarConLineas(PresupuestoID).First();

            decimal beneficio = 0;
            decimal importe = 0;
            decimal coste = 0;
            decimal descuento = 0;

            foreach (LineaPresupuesto linea in presupuesto.LineasPresupuestos)
            {
                importe += linea.Cantidad * linea.Precio;
                coste += linea.Cantidad * linea.Coste;
            }

            if (presupuesto.DescuentoAplicado > 0)
                descuento = importe * presupuesto.DescuentoAplicado / 100;

            beneficio = importe - descuento - coste;

            presupuesto.Importe = (importe - descuento);
            presupuesto.Beneficio = beneficio;
            SaveChanges();
        }

        /// <summary>
        /// Recalcula el presupuesto solicitado.
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <exception cref="System.NotImplementedException"></exception>
        public void Reordenar(int PresupuestoID)
        {
            Presupuesto presupuesto = GetUnPresupuestoSinAceptarConLineas(PresupuestoID).First();

            // lo realizamos en dos pasasadas
            // en la primera le quitamos 1.000.000 al numero para evitar que coincida con ninguno actual
            foreach (LineaPresupuesto linea in presupuesto.LineasPresupuestos.OrderBy(r => r.Orden))
            {
                linea.Orden = linea.Orden - 1000000;
            }
            SaveChanges();

            // y ahora empezamos desde 10
            int orden = 10;

            foreach (LineaPresupuesto linea in presupuesto.LineasPresupuestos.OrderBy(r => r.Orden))
            {
                linea.Orden = orden;
                orden += 10;
            }
            SaveChanges();
        }

        /// <summary>
        /// Aceptar el presupuesto.
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception">Presupuesto no encontrado</exception>
        public int AceptarPresupuesto(int PresupuestoID)
        {
            Presupuesto presupuesto = ObtenerPorID(PresupuestoID);
            if (presupuesto == null)
                throw new Exception("Presupuesto no encontrado");

            presupuesto.Aceptado = true;

            return SaveChanges();
        }

        /// <summary>
        /// Marcar el presupuesto como facturado.
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception">Presupuesto no encontrado</exception>
        public int MarcarComoFacturado(int PresupuestoID)
        {
            Presupuesto presupuesto = ObtenerPorID(PresupuestoID);
            if (presupuesto == null)
                throw new Exception("Presupuesto no encontrado");

            presupuesto.Facturado = true;

            return SaveChanges();
        }

        /// <summary>
        /// Recalcular el presupuesto con los nuevos valores de los productos.
        /// Primero ponemos los nuevos precios y costes a los productos
        /// y luego recalculamos el presupuesto
        /// </summary>
        /// <param name="PresupuestoID">The presupuesto identifier.</param>
        /// <exception cref="System.Exception">Presupuesto no encontrado</exception>
        public void RecalcularTotal(int PresupuestoID)
        {
            Presupuesto presupuesto = GetUnPresupuestoSinAceptarConLineas(PresupuestoID).First();
            if (presupuesto == null)
                throw new Exception("Presupuesto no encontrado");

            Producto producto;

            foreach (LineaPresupuesto linea in presupuesto.LineasPresupuestos)
            {
                // tomamos los datos del producto 
                producto = (new ProductoRepository()).ObtenerPorID((int)linea.ProductoID);
                linea.Precio = producto.Precio;
                linea.Coste = producto.Coste;
            }
            SaveChanges();

            Recalcular(PresupuestoID);

            return;
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
