﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class ProductoRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Nombre">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Producto> GetProductos(string Nombre, string OrdenarPor)
        {
            Nombre = (Nombre == null || Nombre.Trim() == string.Empty) ? string.Empty : Nombre.Trim();

            try
            {
                    DbSet<Producto> query = context.Set<Producto>();
                    if (OrdenarPor == "productoID")
                        return query.Include("Proveedor").Include("TipoProducto").Include("TipoFacturacion").Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.ProductoID).ToList();
                    else if (OrdenarPor == "proveedor")
                        return query.Include("Proveedor").Include("TipoProducto").Include("TipoFacturacion").Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.Proveedor.Denominacion).ToList();
                    else
                        return query.Include("Proveedor").Include("TipoProducto").Include("TipoFacturacion").Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.Nombre).ToList();

                //return query.ToList();
                    //return context.Productos.Where(r => r.Nombre.ToUpper().Contains(Descripcion.ToUpper())).i.OrderBy(r => r.Nombre).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Nombre">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Producto> GetProductos()
        {
            try
            {
                return context.Productos.OrderBy(r => r.Nombre).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Denominacion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Producto> GetProductosActivos()
        {
            try
            {
                return context.Productos.Where(r => r.Activo).OrderBy(r => r.Nombre).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<VistaProductos> GetProductosParaReporte()
        {
            try
            {
                return context.VistaProductos.ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(string Nombre, string DescripcionPresupuesto, int TipoProductoID, int FacturacionID,  decimal Precio, bool Activo, 
            string Observaciones, int? ProveedorID, decimal Coste)
        {
            
            Producto producto = new Producto();
            producto.ProductoID = 0;
            producto.Nombre = Nombre.Trim();
            producto.DescripcionPresupuesto = DescripcionPresupuesto.Trim();
            producto.TipoProductoID = TipoProductoID;
            producto.FacturacionID = FacturacionID;
            producto.Precio = Precio;
            producto.Activo = Activo;
            producto.Observaciones = Observaciones;
            producto.ProveedorID = ProveedorID;
            producto.Coste = Coste;

            context.Productos.Add(producto);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="ProductoID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Producto ObtenerPorID(int ProductoID)
        {
            return context.Productos.Find(ProductoID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="ProductoID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int ProductoID)
        {
            Producto producto = ObtenerPorID(ProductoID);
            if (producto == null)
                return 0;

            context.Productos.Remove(producto);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="ProductoID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int ProductoID, string Nombre, string DescripcionPresupuesto, int TipoProductoID, int FacturacionID, decimal Precio, bool Activo,
            string Observaciones, int? ProveedorID, decimal Coste)
        {
            Producto producto = ObtenerPorID(ProductoID);
            if (producto == null)
                return 0;

            producto.Nombre = Nombre.Trim();
            producto.DescripcionPresupuesto = DescripcionPresupuesto.Trim();
            producto.TipoProductoID = TipoProductoID;
            producto.FacturacionID = FacturacionID;
            producto.Precio = Precio;
            producto.Activo = Activo;
            producto.Observaciones = Observaciones;
            producto.ProveedorID = ProveedorID;
            producto.Coste = Coste;
            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
