﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class ProveedorRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Denominacion">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Proveedor> GetProveedores(string Denominacion, string OrdenarPor)
        {
            Denominacion = (Denominacion == null || Denominacion.Trim() == string.Empty) ? string.Empty : Denominacion.Trim();

            try
            {
                    DbSet<Proveedor> query = context.Set<Proveedor>();
                    if (OrdenarPor == "proveedorID")
                        return query.Where(r => r.Denominacion.ToUpper().Contains(Denominacion.ToUpper()) || Denominacion == string.Empty).OrderBy(r => r.ProveedorID).ToList();
                    else if (OrdenarPor == "activo")
                        return query.Where(r => r.Denominacion.ToUpper().Contains(Denominacion.ToUpper()) || Denominacion == string.Empty).OrderBy(r => r.Activo).ToList();
                    else if (OrdenarPor == "nif")
                        return query.Where(r => r.Denominacion.ToUpper().Contains(Denominacion.ToUpper()) || Denominacion == string.Empty).OrderBy(r => r.NIF).ToList();
                    else
                        return query.Where(r => r.Denominacion.ToUpper().Contains(Denominacion.ToUpper()) || Denominacion == string.Empty).OrderBy(r => r.Denominacion).ToList();

                //return query.ToList();
                    //return context.Proveedores.Where(r => r.Denominacion.ToUpper().Contains(Denominacion.ToUpper())).i.OrderBy(r => r.Denominacion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Denominacion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Proveedor> GetProveedores()
        {
            try
            {
                return context.Proveedores.OrderBy(r => r.Denominacion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Denominacion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Proveedor> GetProveedoresActivos()
        {
            try
            {
                return context.Proveedores.Where(r => r.Activo).OrderBy(r => r.Denominacion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<VistaProveedores> GetProveedoresParaReporte()
        {
            try
            {
                return context.VistaProveedores.ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Denominacion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(string NIF, string Denominacion, bool Activo, bool PersonaFisica, string Direccion, string Codigo_INE, string Codigo_postal, 
            string TelefonoFijo, string TelefonoMovil, string CorreoElectronico, string IBAN, string Observaciones, string PersonaDeContacto)
        {
            
            Proveedor proveedor = new Proveedor();
            proveedor.ProveedorID = 0;
            proveedor.NIF = NIF.ToUpper();
            proveedor.Denominacion = Denominacion.Trim();
            proveedor.Activo = Activo;
            proveedor.PersonaFisica = PersonaFisica;
            proveedor.Direccion = Direccion;
            proveedor.Codigo_INE = Codigo_INE;
            proveedor.Codigo_postal = Codigo_postal;
            proveedor.TelefonoFijo = TelefonoFijo;
            proveedor.TelefonoMovil = TelefonoMovil;
            proveedor.CorreoElectronico = CorreoElectronico;
            proveedor.IBAN = IBAN;
            proveedor.Observaciones = Observaciones;
            proveedor.PersonaDeContacto = PersonaDeContacto;

            context.Proveedores.Add(proveedor);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="ProveedorID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Proveedor ObtenerPorID(int ProveedorID)
        {
            return context.Proveedores.Find(ProveedorID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="ProveedorID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int ProveedorID)
        {
            Proveedor proveedor = ObtenerPorID(ProveedorID);
            if (proveedor == null)
                return 0;

            context.Proveedores.Remove(proveedor);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="ProveedorID">La clave a buscar.</param>
        /// <param name="Denominacion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int ProveedorID, string NIF, string Denominacion, bool Activo, bool PersonaFisica, string Direccion, string Codigo_INE, string Codigo_postal,
            string TelefonoFijo, string TelefonoMovil, string CorreoElectronico, string IBAN, string Observaciones, string PersonaDeContacto)
        {
            Proveedor proveedor = ObtenerPorID(ProveedorID);
            if (proveedor == null)
                return 0;

            proveedor.NIF = NIF.ToUpper(); 
            proveedor.Denominacion = Denominacion.Trim();
            proveedor.Activo = Activo;
            proveedor.PersonaFisica = PersonaFisica;
            proveedor.Direccion = Direccion;
            proveedor.Codigo_INE = Codigo_INE;
            proveedor.Codigo_postal = Codigo_postal;
            proveedor.TelefonoFijo = TelefonoFijo;
            proveedor.TelefonoMovil = TelefonoMovil;
            proveedor.CorreoElectronico = CorreoElectronico;
            proveedor.IBAN = IBAN;
            proveedor.Observaciones = Observaciones;
            proveedor.PersonaDeContacto = PersonaDeContacto;

            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
