﻿using System;
using System.Collections.Generic;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class RubricaRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Rubrica> GetRubricas(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                    if(OrdenarPor=="rubrica")
                        return context.Rubricas.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.RubricaGastoID).ToList();
                    else
                        return context.Rubricas.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();
            
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Rubrica> GetRubricas()
        {
            try
            {
                    return context.Rubricas.OrderBy(r => r.Descripcion).ToList();

            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(string Descripcion)
        {
            Rubrica rubrica = new Rubrica();
            rubrica.RubricaGastoID = 0;
            rubrica.Descripcion = Descripcion;

            context.Rubricas.Add(rubrica);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="RubricaGastoID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Rubrica ObtenerPorID(int RubricaGastoID)
        {
            return context.Rubricas.Find(RubricaGastoID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="RubricaGastoID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int RubricaGastoID)
        {
            Rubrica rubrica = ObtenerPorID(RubricaGastoID);
            if (rubrica == null)
                return 0;

            context.Rubricas.Remove(rubrica);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="RubricaGastoID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int RubricaGastoID, string Descripcion)
        {
            Rubrica rubrica = ObtenerPorID(RubricaGastoID);
            if (rubrica == null)
                return 0;

            rubrica.Descripcion = Descripcion;
            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }
        
        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
