﻿using System;
using System.Collections.Generic;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class TipoFacturacionRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<TipoFacturacion> GetTiposFacturacion(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            try
            {
                if (OrdenarPor == "tipoFacturacion")
                    return context.TiposFacturacion.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.TipoFacturacionID).ToList();
                else
                    return context.TiposFacturacion.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();

            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<TipoFacturacion> GetTiposFacturacion()
        {
            try
            {
                return context.TiposFacturacion.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(string Descripcion)
        {
            TipoFacturacion tipoFacturacion = new TipoFacturacion();
            tipoFacturacion.TipoFacturacionID = 0;
            tipoFacturacion.Descripcion = Descripcion;

            context.TiposFacturacion.Add(tipoFacturacion);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="TipoFacturacionID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public TipoFacturacion ObtenerPorID(int TipoFacturacionID)
        {
            return context.TiposFacturacion.Find(TipoFacturacionID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="TipoFacturacionID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int TipoFacturacionID)
        {
            TipoFacturacion tipoFacturacion = ObtenerPorID(TipoFacturacionID);
            if (tipoFacturacion == null)
                return 0;

            context.TiposFacturacion.Remove(tipoFacturacion);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="TipoFacturacionID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int TipoFacturacionID, string Descripcion)
        {
            TipoFacturacion tipoFacturacion = ObtenerPorID(TipoFacturacionID);
            if (tipoFacturacion == null)
                return 0;

            tipoFacturacion.Descripcion = Descripcion;
            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
