﻿using System;
using System.Collections.Generic;
using System.Linq;

/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class TipoProductoRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Descripcion">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<TipoProducto> GetTiposProducto(string Descripcion, string OrdenarPor)
        {
            Descripcion = (Descripcion == null || Descripcion.Trim() == string.Empty) ? string.Empty : Descripcion.Trim();

            List<TipoProducto> lista;
            try
            {
                if (OrdenarPor == "tipoProducto")
                    lista = context.TiposProducto.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.TipoProductoID).ToList();
                else
                    lista = context.TiposProducto.Where(r => r.Descripcion.ToUpper().Contains(Descripcion.ToUpper()) || Descripcion == string.Empty).OrderBy(r => r.Descripcion).ToList();
                return lista;
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<TipoProducto> GetTiposProducto()
        {
            List<TipoProducto> lista;
            try
            {
                return lista = context.TiposProducto.OrderBy(r => r.Descripcion).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(string Descripcion)
        {
            TipoProducto tipoProducto = new TipoProducto();
            tipoProducto.TipoProductoID = 0;
            tipoProducto.Descripcion = Descripcion;

            context.TiposProducto.Add(tipoProducto);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="TipoProductoID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public TipoProducto ObtenerPorID(int TipoProductoID)
        {
            return context.TiposProducto.Find(TipoProductoID);
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="TipoProductoID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int TipoProductoID)
        {
            TipoProducto tipoProducto = ObtenerPorID(TipoProductoID);
            if (tipoProducto == null)
                return 0;

            context.TiposProducto.Remove(tipoProducto);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="TipoProductoID">La clave a buscar.</param>
        /// <param name="Descripcion">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int TipoProductoID, string Descripcion)
        {
            TipoProducto tipoProducto = ObtenerPorID(TipoProductoID);
            if (tipoProducto == null)
                return 0;

            tipoProducto.Descripcion = Descripcion;
            return SaveChanges();
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

    }
}
