﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
/// <summary>
/// Parte del data layer
/// </summary>
namespace REFOR.DL
{
    /// <summary>
    /// Maneja todas las operaciones de la clase
    /// </summary>
    public class UsuarioRepository : IDisposable
    {
        /// <summary>
        /// El contexto utilizado para interactuar con la base de datos
        /// </summary>
        private ReformasEntities context = new ReformasEntities();

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Nombre">El dato por el que se filtra.</param>
        /// <param name="OrdenarPor">El campo por el que se ordena.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Usuario> GetUsuarios(string Nombre, string OrdenarPor)
        {
            Nombre = (Nombre == null || Nombre.Trim() == string.Empty) ? string.Empty : Nombre.Trim();

            try
            {
                DbSet<Usuario> query = context.Set<Usuario>();
                if (OrdenarPor == "nombre")
                    return query.Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.Nombre).ToList();
                else if (OrdenarPor == "usuarioID")
                    return query.Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.UsuarioID).ToList();
                else
                    return query.Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper()) || Nombre == string.Empty).OrderBy(r => r.Alias).ToList();

                //return query.ToList();
                //return context.Usuarios.Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper())).i.OrderBy(r => r.Nombre).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Obtenemos todos los elementos de la tabla
        /// </summary>
        /// <param name="Nombre">El dato por el que se filtra.</param>
        /// <returns>Lista de elementos solicitados</returns>
        /// <exception cref="System.Exception"></exception>
        public IEnumerable<Usuario> GetUsuarios()
        {
            try
            {
                return context.Usuarios.OrderBy(r => r.Nombre).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }


        /// <summary>
        /// Inserta un nuevo elemento.
        /// </summary>
        /// <param name="Nombre">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Insertar(string Alias, string Nombre, bool Administracion, bool Presupuesto, bool Factura, bool Gasto, bool Contabilidad,
            bool Activo, bool Bloqueado)
        {

            Usuario usuario = new Usuario();
            usuario.UsuarioID = 0;
            usuario.Alias = Alias;
            usuario.Nombre = Nombre.Trim();
            usuario.Clave = Calculo_Clave(Alias);
            usuario.FechaCambioContraseña = DateTime.Now.Date.AddYears(-2);     // la contraseña siempre esta vencida
            usuario.AccesosErroneos = 0;
            usuario.Administracion = Administracion;
            usuario.Presupuesto = Presupuesto;
            usuario.Factura = Factura;
            usuario.Gasto = Gasto;
            usuario.Contabilidad = Contabilidad;
            usuario.Activo = Activo;
            usuario.Bloqueado = Bloqueado;

            context.Usuarios.Add(usuario);
            return SaveChanges();
        }

        /// <summary>
        /// Obtiene un elemento por su identificativo.
        /// </summary>
        /// <param name="UsuarioID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Usuario ObtenerPorID(int UsuarioID)
        {
            return context.Usuarios.Find(UsuarioID);
        }

        /// <summary>
        /// Obtiene un elemento por su alias.
        /// </summary>
        /// <param name="UsuarioID">La clave a buscar.</param>
        /// <returns>Lista de elementos solicitados</returns>
        public Usuario ObtenerPorAlias(string alias)
        {
            try
            {
                DbSet<Usuario> query = context.Set<Usuario>();
                return query.Where(r => r.Alias == alias).OrderBy(r => r.Alias).ToList().ElementAt(0);

                //return query.ToList();
                //return context.Usuarios.Where(r => r.Nombre.ToUpper().Contains(Nombre.ToUpper())).i.OrderBy(r => r.Nombre).ToList();
            }
            catch (Exception ex)
            {

                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Elimina un elemento de la tabla.
        /// </summary>
        /// <param name="UsuarioID">La clave a buscar.</param>
        /// <returns>registros afectados</returns>
        public int Eliminar(int UsuarioID)
        {
            Usuario usuario = ObtenerPorID(UsuarioID);
            if (usuario == null)
                return 0;

            context.Usuarios.Remove(usuario);
            return SaveChanges();
        }

        /// <summary>
        /// Actualiza un elemento de la tabla.
        /// </summary>
        /// <param name="UsuarioID">La clave a buscar.</param>
        /// <param name="Nombre">Campo de la tabla.</param>
        /// <returns>registros afectados</returns>
        public int Actualizar(int UsuarioID, string Alias, string Nombre, DateTime FechaCambioContraseña, bool Administracion, bool Presupuesto, bool Factura, bool Gasto,
            bool Contabilidad, bool Activo, bool Bloqueado)
        {
            Usuario usuario = ObtenerPorID(UsuarioID);
            if (usuario == null)
                return 0;

            usuario.Alias = Alias;
            usuario.Nombre = Nombre.Trim();
            usuario.FechaCambioContraseña = FechaCambioContraseña;
            usuario.Administracion = Administracion;
            usuario.Presupuesto = Presupuesto;
            usuario.Factura = Factura;
            usuario.Gasto = Gasto;
            usuario.Contabilidad = Contabilidad;
            usuario.Activo = Activo;
            usuario.Bloqueado = Bloqueado;

            return SaveChanges();
        }

        /// <summary>
        /// Calcula el hash de la clave introducida
        /// </summary>
        /// <param name="Texto">texto.</param>
        /// <returns>El hash de la clave en hexadecimal</returns>
        private string Calculo_Clave(string Texto)
        {
            // calcula el hash md5 con una longuitud de 16 bytes
            System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
            System.Security.Cryptography.MD5CryptoServiceProvider md5 = new System.Security.Cryptography.MD5CryptoServiceProvider();
            byte[] data = md5.ComputeHash(encoding.GetBytes(Texto));

            StringBuilder sBuilder = new StringBuilder();

            for (int i = 0; i < data.Length; i++)
            {
                sBuilder.Append(data[i].ToString("x2"));
            }

            return sBuilder.ToString();
        }

        /// <summary>
        /// Verifica que la clave y el usuario son correctos
        /// </summary>
        /// <param name="alias">The alias.</param>
        /// <param name="clave">The clave.</param>
        /// <returns>-1 error, 0 password caducado, 1 correcto</returns>
        public int VerificarClave(string alias, string clave)
        {
            Usuario usuario = ObtenerPorAlias(alias);

            int meses = ((new DatoPropioRepository()).ObtenerUnico()).MesesCambioContraseña;

            if (usuario == null)
            {
                return -1;
            }

            if (!usuario.Activo || usuario.Bloqueado || usuario.AccesosErroneos >= 3)
            {
                usuario = null;
                return -1;
            }

            if (usuario.Clave != Calculo_Clave(clave))
            {
                usuario.AccesosErroneos += 1;
                SaveChanges();
                return -1;
            }
            else
            {
                if (usuario.FechaCambioContraseña.AddMonths(meses) < DateTime.Now)
                    return 0;
                else
                {
                    //
                    usuario.AccesosErroneos = 0;
                    usuario.UltimaEntrada = DateTime.Now;
                    SaveChanges();
                    return 1;
                }
            }
        }


        /// <summary>
        /// Cambia la clave de un usuario.
        /// </summary>
        /// <param name="alias">The alias.</param>
        /// <param name="nuevaClave">The nueva clave.</param>
        /// <returns>exito o fracaso</returns>
        public bool CambiarClave(string alias, string nuevaClave)
        {
            try
            {
                Usuario usuario = ObtenerPorAlias(alias);

                usuario.Clave = Calculo_Clave(nuevaClave);
                usuario.AccesosErroneos = 0;
                usuario.FechaCambioContraseña = DateTime.Now;
                usuario.UltimaEntrada = DateTime.Now;
                SaveChanges();
                return true;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Guarda los campos en la tabla.
        /// </summary>
        /// <returns>registros afectados</returns>
        /// <exception cref="System.Exception"></exception>
        public int SaveChanges()
        {
            try
            {
                return context.SaveChanges();
            }
            catch (Exception ex)
            {
                throw new Exception(MensajeError.Crear_mensaje(ex));
            }
        }

        /// <summary>
        /// Hace que todas las claves de usuarios esten vencidas, excepto adm.
        /// </summary>
        /// <returns>exito o fracaso</returns>
        public bool VencerTodasLasClaves()
        {
            try
            {
                foreach (Usuario usuario in GetUsuarios())
                {
                    if (usuario.Alias == "adm")
                        continue;
                    usuario.FechaCambioContraseña.AddYears(-2);
                    SaveChanges();
                }
                return true;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Bloquea todos los usuarios excepto adm.
        /// </summary>
        /// <returns>exito o fracaso</returns>
        public bool BloquearUsurios()
        {
            try
            {
                foreach (Usuario usuario in GetUsuarios())
                {
                    if (usuario.Alias == "adm")
                        continue;
                    usuario.Bloqueado = true;
                    SaveChanges();
                }
                return true;
            }
            catch
            {
                return false;
            }

        }

        /// <summary>
        /// Vencer la clave para un usuario.
        /// </summary>
        /// <param name="alias">The alias.</param>
        /// <returns>exito o fracaso</returns>
        public bool VencerLaClave(string alias)
        {
            try
            {
                Usuario usuario = ObtenerPorAlias(alias);

                usuario.FechaCambioContraseña = usuario.FechaCambioContraseña.AddYears(-2);
                SaveChanges();
                return true;
            }
            catch
            {
                return false;
            }
        }
        
        private bool disposedValue = false;

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposing)
        {
            if (!this.disposedValue)
            {
                if (disposing)
                {
                    context.Dispose();
                }
            }
            this.disposedValue = true;
        }

        /// <summary>
        /// Realiza tareas definidas por la aplicación asociadas a la liberación o al restablecimiento de recursos no administrados.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
