﻿using REFOR.BL;
using REFOR.DL;
using System;
using System.Drawing;
using System.Web.UI;
using System.Web.UI.WebControls;


namespace REFOR.Web.Gestion
{
    public partial class GestionRubricas : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                if (!((Usuario)Session["Usuario"]).Gasto)
                    Response.Redirect(@"~\no_autorizado.aspx");
                txtFilter.Focus();
                Enlazar_datos();
            }

            Page.Master.Page.Form.DefaultButton = btnFilter.UniqueID;

        }

        private void Enlazar_datos()
        {
            try
            {
                GridView1.DataBind();
            }
            catch (Exception err)
            {
                // Message label
                    Master.SetStatus("Se ha producido un error al intentar mostrar los datos de la rejilla. Razón :" + err.Message, Color.Red);
            }
        }

        /// <summary>
        /// Handles the Error event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        public void Page_Error(object sender, EventArgs e)
        {
            Genericas.ErrorInesperado();
        }

        /// <summary>
        /// Handles the PreRenderComplete event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_PreRenderComplete(object sender, EventArgs e)
        {
            Genericas.StampRefreshDetection();
        }

        /// <summary>
        /// Maneja el evento Click event del control btnAñadirRegistro.
        /// Activa el FormView en modo insertar y con los datos en blanco.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.EventArgs"/> que contiene los datos.</param>
        protected void btnAñadirRegistro_Click(object sender, EventArgs e)
        {
            Master.CleanStatus();
            FormView1.Visible = true;
            FormView1.ChangeMode(FormViewMode.Insert);
            GridView1.Visible = false;
            SearchTable.Visible = false;

        }


        /// <summary>
        /// Maneja el evento RowDeleting que se produce antes de borrar un registro.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Web.UI.WebControls.GridViewDeleteEventArgs"/> instance containing the event data.</param>
        protected void GridView1_RowDeleting(object sender, GridViewDeleteEventArgs e)
        {
            // comprobamos si ha sido pulsada la tecla f5 o pulsado el icono de refresco
            if ( Genericas.IsPageRefresh())
            {
                Master.SetStatus("Efectuado refresco de página o sesión caducada", Color.Green);
                e.Cancel = true;
                Enlazar_datos();
                return;
            }
        }

        /// <summary>
        /// Maneja el evento RowDeleted que se produce despues de borrar un registro.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.Web.UI.WebControls.GridViewDeletedEventArgs"/> que contiene los datos.</param>
        protected void GridView1_RowDeleted(object sender, GridViewDeletedEventArgs e)
        {
            if (e.Exception != null)
            {
                if (e.Exception.InnerException != null)
                    Master.SetStatus("Error : No se ha podido borrar el registro </br>" + e.Exception.InnerException.Message, Color.Red);
                else
                    Master.SetStatus("Error : No se ha podido borrar el registro </br>" + e.Exception.Message, Color.Red);
                e.ExceptionHandled = true;
            }
            else
            {
                Enlazar_datos();
                // Message label
                if (e.AffectedRows != 1)
                    Master.SetStatus("Atención. Registro no borrado. Quizas ha sido borrado por otro usuario", Color.Red);
                else
                    Master.SetStatus("Registro borrado", Color.SteelBlue);
            }
        }

        /// <summary>
        /// Maneja el evento SelectedIndexChanged (edición).
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia <see cref="System.EventArgs"/> que contiene los datos.</param>
        protected void GridView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                Master.CleanStatus();
                FormView1.Visible = true;
                FormView1.ChangeMode(FormViewMode.Edit);
                GridView1.Visible = false;
                SearchTable.Visible = false;
            }
            catch (Exception err)
            {
                // Message label
                Master.SetStatus("No se ha podido modificar el registro. Razón :" + err.Message, Color.Red);
            }
        }

        /// <summary>
        /// Permite modificar el formato y comportamiento de la fila de datos 
        /// durante el evento RowDataBound.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.Web.UI.WebControls.GridViewRowEventArgs"/> que contiene el evento.</param>
        protected void GridView1_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            GridViewRowEventArgs ea = e as GridViewRowEventArgs;
            if (ea.Row.RowType == DataControlRowType.DataRow)
            {

                Rubrica rubrica = ea.Row.DataItem as Rubrica;

                ImageButton btnDelete = (ImageButton)e.Row.Cells[0].Controls[0];
                btnDelete.Attributes.Add("onclick", "if(!confirm('Atención, se borrarán los datos de " + Server.HtmlEncode(rubrica.RubricaGastoID.ToString()) + "-" + Server.HtmlEncode(rubrica.Descripcion).Replace("'", "\"") + ".\\n\\n ¿Esta seguro?'))return false");
            }
        }

        /// <summary>
        /// Maneja el evento ItemCommand que se produce despues de pulsar un boton dentro del control FormView.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.Web.UI.WebControls.FormViewCommandEventArgs"/> que contiene los datos.</param>
        protected void FormView1_ItemCommand(object sender, FormViewCommandEventArgs e)
        {
            switch (e.CommandName)
            {
                case "Cancel":
                    Master.SetStatus("Actualización cancelada", Color.SteelBlue);
                    FormView1.ChangeMode(FormViewMode.ReadOnly);
                    FormView1.Visible = false;
                    GridView1.Visible = true;
                    SearchTable.Visible = true;
                    Enlazar_datos();
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Maneja el evento ItemInserting que se produce antes de insertar un registro.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.Web.UI.WebControls.FormViewInsertedEventArgs"/> que contiene los datos.</param>
        protected void FormView1_ItemInserting(object sender, FormViewInsertEventArgs e)
        {
            if (Genericas.IsPageRefresh())
            {
                Master.SetStatus("Efectuado refresco de página o sesión caducada", Color.Green);
                e.Cancel = true;
                BodyTable.Visible = true;
                SearchTable.Visible = true;
                Enlazar_datos();
                return;
            }
        }

        /// <summary>
        /// Maneja el evento ItemInserted que se produce despues de insertar un registro.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.Web.UI.WebControls.FormViewInsertedEventArgs"/> que contiene los datos.</param>
        protected void FormView1_ItemInserted(object sender, FormViewInsertedEventArgs e)
        {
            if (e.Exception != null)
            {
                // Message label
                if (e.Exception.InnerException != null)
                    Master.SetStatus("Error : No se ha podido insertar el registro <br>" + e.Exception.InnerException.Message, Color.Red);
                else
                    Master.SetStatus("Error : No se ha podido insertar el registro <br>" + e.Exception.Message, Color.Red);
                e.ExceptionHandled = true;
                e.KeepInInsertMode = true;
            }
            else
            {
                FormView1.Visible = false;
                GridView1.Visible = true;
                Enlazar_datos();
                SearchTable.Visible = true;
                // Message label
                if (e.AffectedRows != 1)
                    Master.SetStatus("Atención. Registro no añadido.", Color.Red);
                else
                    Master.SetStatus("Registro añadido correctamente", Color.SteelBlue);
            }

        }

        /// <summary>
        /// Maneja el evento ItemUpdating que se produce antes de salvar una modificación de un registro.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.Web.UI.WebControls.FormViewInsertedEventArgs"/> que contiene los datos.</param>
        protected void FormView1_ItemUpdating(object sender, FormViewUpdateEventArgs e)
        {
            if (Genericas.IsPageRefresh())
            {
                Master.SetStatus("Efectuado refresco de página o sesión caducada", Color.Green);
                e.Cancel = true;
                BodyTable.Visible = true;
                SearchTable.Visible = true;
                Enlazar_datos();
                return;
            }
        }

        /// <summary>
        /// Maneja el evento ItemUpdated que se produce despues de salvar una modificación de un registro.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.Web.UI.WebControls.FormViewUpdatedEventArgs"/> que contiene los datos.</param>
        protected void FormView1_ItemUpdated(object sender, FormViewUpdatedEventArgs e)
        {
            if (e.Exception != null)
            {
                // Message label
                if (e.Exception.InnerException != null)
                    Master.SetStatus("Error : No se ha podido modificar el registro </br>" + e.Exception.InnerException.Message, Color.Red);
                else
                    Master.SetStatus("Error : No se ha podido modificar el registro </br>" + e.Exception.Message, Color.Red);
                e.ExceptionHandled = true;
                e.KeepInEditMode = true;
            }
            else
            {
                FormView1.Visible = false;
                GridView1.Visible = true;
                Enlazar_datos();
                SearchTable.Visible = true;
                // Message label
                if (e.AffectedRows != 1)
                    Master.SetStatus("Atención. Modificación no grabada. No existen modificaciones o quizas ha sido borrado por otro usuario", Color.Red);
                else
                    Master.SetStatus("Registro modificado", Color.SteelBlue);
            }
        }


        /// <summary>
        /// Maneja el evento DataBound del control FormView1 que se produce despues de asignar el valor a los campos.
        /// Permite asignar valores a campos que vienen en blanco o bien darles un cierto formato.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.EventArgs"/> que contiene los datos.</param>
        protected void FormView1_DataBound(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Aplica el filtro solicitado.
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.EventArgs"/>  que contiene los datos.</param>
        protected void btnFilter_Click(object sender, EventArgs e)
        {
            try
            {
                Master.CleanStatus();
                Enlazar_datos();
            }
            catch (Exception err)
            {
                // Message label
                if (err.InnerException != null && err.InnerException.Message != string.Empty)
                    Master.SetStatus("Se ha producido un error al intentar filtrar los datos de la rejilla. Razón :" + err.InnerException.Message, Color.Red);
                else
                    Master.SetStatus("Se ha producido un error al intentar filtrar los datos de la rejilla. Razón :" + err.Message, Color.Red);
            }
        }

        /// <summary>
        /// Maneja el evento PreRender que se produce despues de haber creado el control pero antes de mostrarlo en pantalla.
        /// Permite cambiar de forma drastica la forma de presentar los datos del gridview como por ejemplo añadiendole una
        /// nueva fila de header
        /// </summary>
        /// <param name="sender">La fuente del evento.</param>
        /// <param name="e">La instancia de <see cref="System.EventArgs"/> que contiene los datos.</param>
        protected void GridView1_PreRender(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Maneja el evento _deleted que se produce despues de haber efectuado el borrado.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Web.UI.WebControls.ObjectDataSourceStatusEventArgs"/> instance containing the event data.</param>
        protected void odsGeneral_Deleted(object sender, ObjectDataSourceStatusEventArgs e)
        {
            if (e.ReturnValue != null)
                e.AffectedRows = (int)e.ReturnValue;
        }

        /// <summary>
        /// Maneja el evento _updated que se produce despues de haber efectuado la modificacion
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Web.UI.WebControls.ObjectDataSourceStatusEventArgs"/> instance containing the event data.</param>
        protected void odsDetalle_Updated(object sender, ObjectDataSourceStatusEventArgs e)
        {
            if (e.ReturnValue != null)
                e.AffectedRows = (int)e.ReturnValue;
        }

        /// <summary>
        /// Maneja el evento _Inserted que se produce despues de haber efectuado la insercion.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Web.UI.WebControls.ObjectDataSourceStatusEventArgs"/> instance containing the event data.</param>
        protected void odsDetalle_Inserted(object sender, ObjectDataSourceStatusEventArgs e)
        {
            if (e.ReturnValue != null)
                e.AffectedRows = (int)e.ReturnValue;
        }
    }
}
